@@ -2,9 +2,9 @@
 #
 # plot.R
 #
-# copyright (c) 2000-2015, Karl W Broman
+# copyright (c) 2000-2019, Karl W Broman
 #       [modifications of plot.cross from Brian Yandell]
-# last modified Aug, 2015
+# last modified Dec, 2019
 # first written Mar, 2000
 #
 #     This program is free software; you can redistribute it and/or
@@ -25,12 +25,12 @@
 #
 ######################################################################
 
-plotMissing <- plot.missing <-
+plotMissing <-
     function(x, chr, reorder=FALSE, main="Missing genotypes",
              alternate.chrid=FALSE, ...)
 {
     cross <- x
-    if(!any(class(cross) == "cross"))
+    if(!inherits(cross, "cross"))
         stop("Input should have class \"cross\".")
     if(!missing(chr)) cross <- subset(cross,chr=chr)
 
@@ -58,7 +58,7 @@ plotMissing <- plot.missing <-
     # make matrix with  0 where genotype data is missing
     #                   1 where data is not missing
     #                 0.5 where data is partially missing
-    type <- class(cross)[1]
+    type <- crosstype(cross)
     g <- t(Geno[o,])
     g[is.na(g)] <- 0
     if(type == "bc" || type=="risib" || type=="riself" || type=="bc")
@@ -127,20 +127,20 @@ plotMissing <- plot.missing <-
 
 geno.image <-
     function(x, chr, reorder=FALSE, main="Genotype data",
-             alternate.chrid=FALSE, ...)
+             alternate.chrid=FALSE, col=NULL, ...)
 {
     cross <- x
-    if(!any(class(cross) == "cross"))
+    if(!inherits(cross, "cross"))
         stop("Input should have class \"cross\".")
     if(!missing(chr)) cross <- subset(cross,chr=chr)
 
-    type <- class(cross)[1]
+    type <- crosstype(cross)
 
     # revise X chromosome data
     if(type=="bc" || type=="f2") {
-        chrtype <- sapply(cross$geno, class)
-        if(any(chrtype=="X")) {
-            for(i in which(chrtype=="X"))
+        chr_type <- sapply(cross$geno, chrtype)
+        if(any(chr_type=="X")) {
+            for(i in which(chr_type=="X"))
                 cross$geno[[i]]$data <- reviseXdata(type, "simple", getsex(cross),
                                                     geno=cross$geno[[i]]$data,
                                                     cross.attr=attributes(cross))
@@ -169,6 +169,15 @@ geno.image <-
     }
     thecolors <- thecolors[1:(maxgeno+1)]
     thebreaks <- thebreaks[1:(maxgeno+2)]
+    if(!is.null(col)) { # colors provided
+        if(length(col) < length(thecolors)) {
+            warning("col should have length ", maxgeno+1)
+            thecolors[seq_along(col)] <- col
+        }
+        else {
+            thecolors <- col[1:(maxgeno+1)]
+        }
+    }
 
     # reorder the individuals according to their phenotype
     o <- 1:nrow(Geno)
@@ -176,7 +185,7 @@ geno.image <-
         # if reorder is a number, use the corresponding phenotype
         if(is.numeric(reorder)) {
             if(reorder < 1 || reorder > nphe(cross))
-                stop("reorder should be TRUE, FALSE, or an integer between 1 and", nphe(cross))
+                stop("reorder should be TRUE, FALSE, or an integer between 1 and ", nphe(cross))
 
             o <- order(cross$pheno[,reorder])
         }
@@ -293,12 +302,12 @@ plotMap <- plot.map <-
     map <- x
     # figure out if the input is a cross (containing a map)
     #    or is the map itself
-    if(any(class(map) == "cross"))
+    if(inherits(map, "cross"))
         map <- pull.map(map)
-    if(!missing(map2) && any(class(map2) == "cross"))
+    if(!missing(map2) && inherits(map2, "cross"))
         map2 <- pull.map(map2)
 
-    if(!any(class(map)=="map")  || (!missing(map2) && !any(class(map2) == "map")))
+    if(!inherits(map, "map")  || (!missing(map2) && !inherits(map2, "map")))
         warning("Input should have class \"cross\" or \"map\".")
 
     if(!missing(map2) && is.matrix(map[[1]]) != is.matrix(map2[[1]]))
@@ -622,14 +631,14 @@ plot.cross <-
               alternate.chrid=TRUE, ...)
 {
     # look to see whether this should really be shipped to plotMap
-    if("map" %in% class(auto.layout) &&
-       ("map" %in% class(x) || "cross" %in% class(x))) {
+    if(inherits(auto.layout, "map") &&
+       (inherits(x, "map") || inherits(x, "cross"))) {
         plotMap(x, auto.layout, alternate.chrid=alternate.chrid, ...)
         return(invisible())
     }
 
     # make sure this is a cross
-    if(!("cross" %in% class(x)))
+    if(!inherits(x, "cross"))
         stop("Input should have class \"cross\".")
 
     old.yaxt <- par("yaxt")
@@ -672,12 +681,12 @@ plot.cross <-
 #
 ######################################################################
 
-plotGeno <- plot.geno <-
+plotGeno <-
     function(x, chr, ind, include.xo=TRUE, horizontal=TRUE,
              cutoff=4, min.sep=2, cex=1.2, ...)
 {
     cross <- x
-    if(!any(class(cross) == "cross"))
+    if(!inherits(cross, "cross"))
         stop("Input should have class \"cross\".")
 
     if(missing(chr)) chr <- names(cross$geno)[1]
@@ -694,7 +703,7 @@ plotGeno <- plot.geno <-
     if(is.null(id)) id <- 1:nind(cross)
     use.id <- TRUE
 
-    type <- class(cross)[1]
+    type <- crosstype(cross)
 
     old.las <- par("las")
     on.exit(par(las=old.las))
@@ -731,8 +740,8 @@ plotGeno <- plot.geno <-
 
     # revise X chr data for backcross/intercross
     data <- cross$geno[[1]]$data
-    chrtype <- class(cross$geno[[1]])
-    if(chrtype=="X" && (type=="f2" || type=="bc"))
+    chr_type <- chrtype(cross$geno[[1]])
+    if(chr_type=="X" && (type=="f2" || type=="bc"))
         data <- reviseXdata(type, sexpgm=getsex(cross), geno=data, cross.attr=attributes(cross), force=TRUE)
 
     if(include.xo) {
@@ -743,7 +752,7 @@ plotGeno <- plot.geno <-
         }
         else { # 4-way cross
             mcross <- dcross <- cross
-            class(mcross)[1] <- class(dcross)[1] <- "bc"
+            class(mcross) <- class(dcross) <- c("bc", "cross")
             mcross$geno[[1]]$data[!is.na(data) & data==1 | data==3 | data==5] <- 1
             mcross$geno[[1]]$data[!is.na(data) & data==2 | data==4 | data==6] <- 2
             mcross$geno[[1]]$data[!is.na(data) & data==7 | data==8 | data==9 | data==10] <- NA
@@ -966,11 +975,11 @@ plotGeno <- plot.geno <-
 
             # AB genotypes
             ind <- tind; ind[!is.na(data) & data!=2] <- NA
-            if(type=="f2" || (type=="bc" && chrtype=="X"))
+            if(type=="f2" || (type=="bc" && chr_type=="X"))
                 points(x,ind,pch=21,col="black", bg=color[2],cex=cex)
             else points(x,ind,pch=21,col="black", bg=color[3],cex=cex)
 
-            if(type=="f2" || (type=="bc" && chrtype=="X")) {
+            if(type=="f2" || (type=="bc" && chr_type=="X")) {
                 # BB genotypes
                 ind <- tind; ind[!is.na(data) & data!=3] <- NA
                 points(x,ind,pch=21,col="black", bg=color[3],cex=cex)
@@ -1016,11 +1025,11 @@ plotGeno <- plot.geno <-
 
             # AB genotypes
             ind <- tind; ind[!is.na(data) & data!=2] <- NA
-            if(type=="f2" || (type=="bc" && chrtype=="X"))
+            if(type=="f2" || (type=="bc" && chr_type=="X"))
                 points(ind,y,pch=21,col="black", bg=color[2],cex=cex)
             else points(ind,y,pch=21,col="black", bg=color[3],cex=cex)
 
-            if(type=="f2" || (type=="bc" && chrtype=="X")) {
+            if(type=="f2" || (type=="bc" && chr_type=="X")) {
                 # BB genotypes
                 ind <- tind; ind[!is.na(data) & data!=3] <- NA
                 points(ind,y,pch=21,col="black", bg=color[3],cex=cex)
@@ -1059,7 +1068,7 @@ plotGeno <- plot.geno <-
 #            genotype data.
 #
 ######################################################################
-plotInfo <- plot.info <-
+plotInfo <-
     function(x,chr,method=c("entropy","variance","both"), step=1,
              off.end=0, error.prob=0.001,
              map.function=c("haldane","kosambi","c-f","morgan"),
@@ -1067,7 +1076,7 @@ plotInfo <- plot.info <-
              include.genofreq=FALSE, ...)
 {
     cross <- x
-    if(!any(class(cross) == "cross"))
+    if(!inherits(cross, "cross"))
         stop("Input should have class \"cross\".")
 
     method <- match(match.arg(method),c("entropy","variance","both"))-1
@@ -1162,8 +1171,8 @@ plotInfo <- plot.info <-
 
         if(include.genofreq) {
             p <- cross$geno[[i]]$prob
-            if(class(cross$geno[[i]])=="X")
-                p <- reviseXdata(class(cross)[1], expandX="full", sexpgm=getsex(cross), prob=p,
+            if(chrtype(cross$geno[[i]])=="X")
+                p <- reviseXdata(crosstype(cross), expandX="full", sexpgm=getsex(cross), prob=p,
                                  cross.attr=attributes(cross))
             p <- apply(p, 2:3, mean, na.rm=TRUE)
             if(is.null(theprob))
@@ -1292,14 +1301,14 @@ plotInfo <- plot.info <-
 
 
 # plot phenotypes against one or more markers
-plotPXG <- plot.pxg <-
+plotPXG <-
     function(x, marker, pheno.col = 1, jitter = 1, infer = TRUE,
              pch, ylab, main, col, ...)
 {
     cross <- x
-    if(!any(class(cross) == "cross"))
+    if(!inherits(cross, "cross"))
         stop("Input should have class \"cross\".")
-    type <- class(cross)[1]
+    type <- crosstype(cross)
 
     if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
         cross$pheno <- cbind(pheno.col, cross$pheno)
@@ -1348,19 +1357,19 @@ plotPXG <- plot.pxg <-
     map <- pull.map(cross)
     pos <- NULL
     for(i in seq(length(chr))) pos[i] <- map[[chr[i]]][marker[i]]
-    chrtype <- sapply(cross$geno, class)
-    names(chrtype) <- names(cross$geno)
-    chrtype <- chrtype[chr]
+    chr_type <- sapply(cross$geno, chrtype)
+    names(chr_type) <- names(cross$geno)
+    chr_type <- chr_type[chr]
 
     # if X chromosome and backcross or intercross, get sex/direction data
-    if(any(chrtype == "X") && (type == "bc" || type == "f2"))
+    if(any(chr_type == "X") && (type == "bc" || type == "f2"))
         sexpgm <- getsex(cross)
     else sexpgm <- NULL
 
     # number of possible genotypes
     gen.names <- list()
     for(i in seq(length(chr)))
-        gen.names[[i]] <- getgenonames(type, chrtype[i], "full", sexpgm, attributes(cross))
+        gen.names[[i]] <- getgenonames(type, chr_type[i], "full", sexpgm, attributes(cross))
     n.gen <- sapply(gen.names, length)
 
     jitter <- jitter/10
@@ -1404,8 +1413,8 @@ plotPXG <- plot.pxg <-
     }
 
     # in case of X chromosome, recode some genotypes
-    if(any(chrtype == "X") && (type == "bc" || type == "f2")) {
-        ix = seq(n.mark)[chrtype == "X"]
+    if(any(chr_type == "X") && (type == "bc" || type == "f2")) {
+        ix = seq(n.mark)[chr_type == "X"]
         for(i in ix)
             x[, i] <- as.numeric(reviseXdata(type, "full", sexpgm,
                                              geno = as.matrix(x[, i]),
@@ -1449,7 +1458,8 @@ plotPXG <- plot.pxg <-
 
     # create plot
     plot(x + u, y, xlab = "Genotype", ylab = ylab, type = "n",
-         main = "", xlim = c(1 - r + jitter, length(gnames) + r + jitter), xaxt = "n")
+         main = "", xlim = c(1 - r + jitter, length(gnames) + r + jitter), xaxt = "n",
+         ...)
 
     # marker names at top
     if(missing(main))
@@ -1507,10 +1517,10 @@ plotPXG <- plot.pxg <-
     invisible(data)
 }
 
-plotPheno <- plot.pheno <-
+plotPheno <-
     function(x, pheno.col=1, ...)
 {
-    if(!any(class(x) == "cross"))
+    if(!inherits(x, "cross"))
         stop("Input should have class \"cross\".")
 
     if(LikePheVector(pheno.col, nind(x), nphe(x))) {
